{% extends "admin/base_site.html" %}

{% block title %}Ooooooops! Something went wrong{% endblock %}

{% block extrahead %}
        <style type="text/css">
            <!--
            div.scroll {
            height: 500px;
            width: 90%;
            overflow: auto;
            border: 1px dotted #666;
            background-color: #EEEEEE;
            padding: 8px;
            }
            -->
        </style> 
{% endblock %}

{% block content %}

    <body onload="document.getElementById('logs').scrollTop = document.getElementById('logs').scrollHeight">
        <h1>Ooooooops! Something went wrong</h1>
	<p />
        <div class="scroll" id="logs"><pre>{% for i in log %}{{ i }}{% endfor %}</pre></div>
        <p /><p />
	<button onclick="javascript:history.go(-1)">Go back to admin interface</button>
    </body>


{% endblock %}

