{% extends "admin/base.html" %}
{% load version %}
{% load i18n %}

{% block title %}{{ title }} | {% trans 'Django PKI administration' %}{% endblock %}

{% block branding %}
<h1 id="site-name">{% trans 'Django PKI administration' %}</h1>
{% endblock %}

{% block footer %}<div style="text-align: center; font-style: italic; font-size: 10px; margin-bottom: 20px;">django-pki - v{% pki_version %}</div>{% endblock %}
